
namespace eval History {
    variable list
    variable pos
    variable menu

    proc Add { name } {
	variable list
	variable pos
	variable menu

	lappend list $name
	set pos [expr [llength $list]-1]
	if { $pos == 0 } {
	    $menu entryconf Backward -state disabled
	} else {
	    $menu entryconf Backward -state normal
	}
	$menu entryconf Forward -state disabled
    }

    proc GoBackward { w } {
	variable list
	variable pos
	variable menu

	incr pos -1
	if { $pos == 0 } {
	    $menu entryconf Backward -state disabled
	}
	$menu entryconf Forward -state normal
	LoadRef $w [lindex $list $pos] 0
    }
    proc GoForward { w } {
	variable list
	variable pos
	variable menu
	
	incr pos 1
	if { $pos == [expr [llength $list]-1] } {
	    $menu entryconf Forward -state disabled
	}
	$menu entryconf Backward -state normal
	LoadRef $w [lindex $list $pos] 0
    }
}

namespace eval HelpPrefs {
    variable tttrick 0
}

namespace eval HelpViewer {

    variable LastFileList

    proc GiveLastFile { w } {
	variable LastFileList
	set retval ""
	catch {
	    set w [winfo toplevel $w]
	    set retval $LastFileList($w)
	}
	return $retval
    }
    proc EnterLastFile { w file } {
	variable LastFileList
	set w [winfo toplevel $w]
	set LastFileList($w) $file
    }

    #A font chooser routine.
    #
    #  $base.h.h config -fontcommand pickFont
    proc pickFont {size attrs} { 
	global tcl_platform
	#      puts "FontCmd: $size $attrs"

	if { $HelpPrefs::tttrick } {
	    set a [expr {-1<[lsearch $attrs fixed]?{Symbol}:{Helvetica}}]
	} else {
	    set a [expr {-1<[lsearch $attrs fixed]?{Courier}:{Helvetica}}]
	}
	set b [expr {-1<[lsearch $attrs italic]?{italic}:{roman}}]
	set c [expr {-1<[lsearch $attrs bold]?{bold}:{normal}}]
	if { $tcl_platform(platform) != "windows"} {
	    set d [expr {3+int(12*pow(1.2,$size-4))}]
	} else {
	    set d [expr {3+int(12*pow(1.2,$size-6))}]
	}
	list $a $d $b $c
    } 

    proc CreateFrame { frame { hscroll 1 } {vscroll 1 }  } {
	frame $frame
	frame $frame.f1
	frame $frame.f2
	if { $hscroll } {
	    set hscrollcommand "$frame.f2hsb set"
	} else { set xscrollcommand "" }
	if { $vscroll } {
	    set yscrollcommand "$frame.f1vsb set"
	} else { set yscrollcommand "" }
	html $frame.f1.h \
		-xscrollcommand $hscrollcommand \
		-yscrollcommand $yscrollcommand \
		-padx 5 \
		-pady 9 \
		-formcommand FormCmd \
		-imagecommand ImageCmd \
		-scriptcommand ScriptCmd \
		-appletcommand AppletCmd \
		-underlinehyperlinks 0 \
		-bg white -tablerelief raised \
		-resolvercommand ResolveUri \
		-exportselection 1 \
		-takefocus 1

	bind $frame.f1.h.x <1> "HelpViewer::HrefBinding $frame.h %x %y"
	if { $hscroll } {
	    frame $frame.f2.sp -width [winfo reqwidth $base.h.vsb] -bd 2 -relief raised
	    scrollbar $frame.f2.hsb -orient horizontal -command "$frame.f1.h xview"
	    pack $base.f2.sp -side right -fill y
	    pack $frame.f2.hsb -side left -fill x -expand 1
	}
	if { $vscroll } {
	    scrollbar $frame.f1.vsb -orient vertical -command "$frame.f1.h yview"
	    pack $frame.f1.vsb -side right -fill y -expand 1
	}
	pack $frame.f1.h -side left -fill both -expand 1
	pack $frame.f1 -side top -fill both -expand 1
	pack $frame.f2 -side bottom -fill x -expand 1
    }

    proc FrameCmd { base type arglist } {
	global HelpPriv
	switch $type {
	    frameset {
		set HelpPriv(frameset) $arglist
		foreach i [array names frame*] {
		    uset HelpPriv($i)
		}
	    }
	    frame {
		set i 0
		while { [info exists HelpPriv(frame$i)] } { incr i }
		set HelpPriv(frame$i) $arglist
	    }
	    /frameset {
		foreach "name args" "$HelpPriv(frameset)" {
		    switch $name {
		        rows {
		        }
		    }
		}
		set i 0
		while { [info exists HelpPriv(frame$i)] } { 
		    foreach "name args" "$HelpPriv(frame$i)" {


		    }
		    incr i
		}
	    }
	}
	tk_messageBox -icon error -message $type----$arglist-- -type ok
    }

    # This routine is called for each form element
    #
    proc FormCmd {n cmd args} {
	# puts "FormCmd: $n $cmd $args"
	switch $cmd {
	    select -
	    textarea -
	    input {
		set w [lindex $args 0]
		label $w -image nogifsm
	    }
	}
    }

    proc ImageCmd {args} {
	global OldImages Images showImages
	if {!$showImages} {
	    return smgray
	}
	set fn [lindex $args 0]

	set list [file split $fn]
	if { [lindex $list 0] == "." } {
	    set list [lrange $list 1 end]
	}
	set fn [eval [concat "file join $list"]]

	if { [file dirname $fn] == "." } {
	    set fn [file tail $fn]
	}
	if {[info exists OldImages($fn)]} {
	    set Images($fn) $OldImages($fn)
	    unset OldImages($fn)
	    return $Images($fn)
	}
	if {[catch {image create photo -file $fn} img]} {
	    return smgray
	}
	if {[image width $img]*[image height $img]>20000} {
	    global BigImages
	    set b [image create photo -width [image width $img] \
		    -height [image height $img]]
	    set BigImages($b) $img
	    set img $b
	    after idle "HelpViewer::MoveBigImage $b"
	}
	set Images($fn) $img
	return $img
    }
    proc MoveBigImage b {
	global BigImages
	if {![info exists BigImages($b)]} return
	$b copy $BigImages($b)
	image delete $BigImages($b)
	unset BigImages($b)
	update
    }


    # This routine is called for every <SCRIPT> markup
    #
    proc ScriptCmd {args} {
	# puts "ScriptCmd: $args"
    }

    # This routine is called for every <APPLET> markup
    #
    proc AppletCmd {w arglist} {
	# puts "AppletCmd: w=$w arglist=$arglist"
	label $w -text "The Applet $w" -bd 2 -relief raised
    }

    # This procedure is called when the user clicks on a hyperlink.
    # See the "bind $base.h.h.x" below for the binding that invokes this
    # procedure
    #
    proc HrefBinding {w x y} {
	set new [$w href $x $y]
	LoadRef $w $new
    }
    proc LoadRef { w new { enterinhistory 1 } } {
	global tcl_platform

	if { $new != "" && [regexp {[a-zA-Z]+[a-zA-Z]:.*} $new] } {
	    regexp {[^/\\]*:.*} $new url
	    if { [regexp {:/[^/]} $url] } {
		regsub {:/} $url {://} url
	    }
	    if { $tcl_platform(platform) != "windows"} {
		set comm [auto_execok netscape]
		if { $comm == "" } {
		    tk_messageBox -icon warning -message \
		            "Check url: $url in your web browser." -type ok
		} else {
		    exec $comm $url &
		}
	    } else {
		set comm [freewrap::shell_getCmd_imp .html open]
		if { $comm == "" } {
		    tk_messageBox -icon warning -message \
		            "Check url: $url in your web browser." -type ok
		} else {
		    regsub -all {\\} $comm {\\\\} comm
		    eval [concat "exec $comm $url &"]
		}
	    }
	    return
	}

	if {$new!=""} {
	    set LastFile [GiveLastFile $w]
	    if { [string match #* [file tail $new]] } {
		set new $LastFile[file tail $new]
	    }
	    set pattern $LastFile#
	    set len [string length $pattern]
	    incr len -1
	    if {[string range $new 0 $len]==$pattern} {
		incr len
		$w yview [string range $new $len end]
		if { $enterinhistory } {
		    History::Add $new
		}
	    } elseif { [regexp {(.*)#(.*)} $new {} file tag] } {
		LoadFile $w $file $enterinhistory $tag
	    } else {
		LoadFile $w $new $enterinhistory
	    }
	}
    }

    proc Load { w } {
	set filetypes {
	    {{Html Files} {.html .htm}}
	    {{All Files} *}
	}
	global lastDir htmltext
	set f [tk_getOpenFile -initialdir $lastDir -filetypes $filetypes]
	if {$f!=""} {
	    LoadFile $w $f
	    set lastDir [file dirname $f]
	}
    }

    # Clear the screen.
    #
    # Clear the screen.
    #
    proc Clear { w } {
	global Images OldImages hotkey
	$w clear
	catch {unset hotkey}
#          ClearBigImages
#          ClearOldImages
#          foreach fn [array names Images] {
#              set OldImages($fn) $Images($fn)
#          }
#          catch {unset Images}
    }
    proc ClearOldImages {} {
	global OldImages
#          foreach fn [array names OldImages] {
#              image delete $OldImages($fn)
#          }
#          catch {unset OldImages}
    }
    proc ClearBigImages {} {
	global BigImages
#          foreach b [array names BigImages] {
#              image delete $BigImages($b)
#          }
#          catch {unset BigImages}
    }

    # Read a file
    #
    proc ReadFile {name} {
	if { [file dirname $name] == "." } {
	    set name [file tail $name]
	}
	if {[catch {open $name r} fp]} {
	    tk_messageBox -icon error -message $fp -type ok
	    return {}
	} else {
	    fconfigure $fp -translation binary
	    set r [read $fp [file size $name]]
	    close $fp
	    return $r
	}
    }

    # Load a file into the HTML widget
    #
    proc LoadFile {w name { enterinhistory 1 } { tag "" } } {
	global HelpPriv

	if { $name == "" } { return }
	set html [ReadFile $name]
	if {$html==""} return
	Clear $w
	EnterLastFile $w $name

	$w config -base $name
	if { $enterinhistory } {
	    if { $tag == "" } {
		History::Add $name
	    } else {
		History::Add $name#$tag
	    }
	}
	$w sel clear
	$w parse $html
	ClearOldImages
	if { $tag != "" } {
	    update idletasks
	    $w yview $tag
	}
    }

    # Refresh the current file.
    #
    proc Refresh {w args} {
	set LastFile [GiveLastFile $w]
	if {![info exists LastFile] || ![winfo exists $w] } return
	LoadFile $w $LastFile 0
    }

    proc ResolveUri { args } {
	return [file join [file dirname [lindex $args 0]] [lindex $args 1]]
    }

    proc ManageSel { htmlw w x y type } {
	global HelpPriv tcl_platform
	if { ![winfo exists  $w] } { return }
	if { [winfo parent $w] != $htmlw } { return }
	if { [info exists HelpPriv(lastafter)] } {
	    after cancel $HelpPriv(lastafter)
	}
	switch -glob $type {
	    press {
		update idletasks
		$htmlw sel clear
		set idx [$htmlw index @$x,$y]
		if { $idx == "" } { return }
		$htmlw sel set $idx $idx
	    }
	    motion* {
		if { $type == "motion" } {
		    set ini [$htmlw index sel.first]
		    if { $ini == "" } { 
		        $htmlw sel clear
		    } else {
		        set idx [$htmlw index @$x,$y]
		        $htmlw sel set $ini $idx
		    }
		}
		set isout 0
		if { $x > [winfo width $htmlw] } {
		    $htmlw xview scroll 1 units
		    set isout 1
		} elseif { $x <0 } {
		    $htmlw xview scroll -1 units
		    set isout 1
		}
		if { $y > [winfo height $htmlw] } {
		    $htmlw yview scroll 1 units
		    set isout 1
		} elseif { $y <0 } {
		    $htmlw yview scroll -1 units
		    set isout 1
		}
		if { $isout } {
		    set HelpPriv(lastafter) [after 100 ManageSel \
		            $htmlw $w $x $y motionout]
		}
	    }
	    release {
		set ini [$htmlw index sel.first]
		if { $ini == "" } { 
		    $htmlw sel clear
		    return
		}
		set idx [$htmlw index @$x,$y]
		if { $idx == $ini } {
		    $htmlw sel clear
		} else {
		    $htmlw sel set $ini $idx
		}
		if { $tcl_platform(platform) != "windows"} {
		    selection own -command "LooseSelection $htmlw" $htmlw
		}
	    }
	    
	}
    }


    proc LooseSelection { w } {
	$w sel clear
    }

    proc CopySelected { w { offset 0 } { maxBytes 0} } {
	global tcl_platform

	set ini [$w index sel.first]
	set end [$w index sel.last]
	if { $ini == "" || $end == "" } { return }

	regexp {([0-9]+)[.]([0-9]+)} $ini {} initoc inipos
	regexp {([0-9]+)[.]([0-9]+)} $end {} endtoc endpos
	#puts [$w token list $initoc $endtoc]
	set rettext ""
	set iposlast [expr $endtoc-$initoc]
	set ipos 0
	foreach i [$w token list $initoc $endtoc] {
	    set type [lindex $i 0]
	    set contents [lrange $i 1 end]
	    switch -- $type {
		Text {
		    set inichar 0
		    set endchar [string length $contents]
		    if { $ipos == 0 } {
		        set inichar $inipos
		    }
		    if { $ipos == $iposlast } {
		        set endchar [expr $endpos-1]
		    }
		    append rettext [string range $contents $inichar $endchar]
		}
		Space {
		    if { [lindex $contents 1] == 0 } {
		        append rettext " "
		    } else {
		        append rettext "\n"
		    }
		}
	    }
	    incr ipos
	}
	if { $tcl_platform(platform) == "windows"} {
	    clipboard clear -displayof $w
	    clipboard append -displayof $w $rettext
	} else {
	    return [string range $rettext $offset [expr $offset+$maxBytes-1]]
	}
    }

    proc HelpWindow { file { base .help} { geom "" } { title "" } } {
	global lastDir GIDDEFAULT tcl_platform argv0

	if { ![info exists GIDDEFAULT] } {
	    set GIDDEFAULT .
	}

	if { $tcl_platform(platform) != "windows" } {
	    option add *Scrollbar*Width 10
	    option add *Scrollbar*BorderWidth 1
	    option add *Button*BorderWidth 1
	}
	option add *Menu*TearOff 0
	
	if { $tcl_platform(platform) != "windows" } {
	    option add *background AntiqueWhite3
	    option add *Button*background bisque3
	    option add *Menu*background bisque3
	    option add *Button*foreground black
	    option add *Entry*background thistle
	    option add *DisabledForeground grey60
	    option add *HighlightBackground AntiqueWhite3
	}

	catch { destroy $base }

	if { $title == "" } { set title Help }


#          if { $geom == "" } {
#              set width 400
#              set height 500
#              set x [expr [winfo screenwidth $base]/2-$width/2]
#              set y [expr [winfo screenheight $base]/2-$height/2]
#              wm geom $base ${width}x${height}+${x}+${y}
#          } else {
#              wm geom $base $geom
#          }

	if { [info procs InitWindow] != "" } {
	    InitWindow $base $title PostHelpViewerWindowGeom
	} else {
	    toplevel $base
	    wm title $base $title
	}


	# Make sure the html widget is loaded into
	# our interpreter
	#
	if {[info command html]==""} {
	    #if {![package require tkhtml]} return
	    foreach f [list \
		    $GIDDEFAULT/tkhtml.so\
		    ./tkhtml.so\
		    [file join [file dirname $argv0] tkhtml.so] \
		    [file join [file dirname $argv0] tkhtml.dll] \
		    $GIDDEFAULT/tkhtml.dll\
		    ./tkhtml.dll\
		    ] {
		if {[file exists $f]} {
		    if {[catch {load $f Tkhtml}]==0} break
		}
	    }
	}

	# These images are used in place of GIFs or of form elements
	#

	if { [lsearch [image names] biggray] == -1 } {
	    
	    image create photo biggray -data {
		R0lGODdhPAA+APAAALi4uAAAACwAAAAAPAA+AAACQISPqcvtD6OctNqLs968+w+G4kiW5omm
		6sq27gvH8kzX9o3n+s73/g8MCofEovGITCqXzKbzCY1Kp9Sq9YrNFgsAO///
	    }
	    image create photo smgray -data {
		R0lGODdhOAAYAPAAALi4uAAAACwAAAAAOAAYAAACI4SPqcvtD6OctNqLs968+w+G4kiW5omm
		6sq27gvH8kzX9m0VADv/
	    }
	    image create photo nogifbig -data {
		R0lGODdhJAAkAPEAAACQkADQ0PgAAAAAACwAAAAAJAAkAAACmISPqcsQD6OcdJqKM71PeK15
		AsSJH0iZY1CqqKSurfsGsex08XuTuU7L9HywHWZILAaVJssvgoREk5PolFo1XrHZ29IZ8oo0
		HKEYVDYbyc/jFhz2otvdcyZdF68qeKh2DZd3AtS0QWcDSDgWKJXY+MXS9qY4+JA2+Vho+YPp
		FzSjiTIEWslDQ1rDhPOY2sXVOgeb2kBbu1AAADv/
	    }
	    image create photo nogifsm -data {
		R0lGODdhEAAQAPEAAACQkADQ0PgAAAAAACwAAAAAEAAQAAACNISPacHtD4IQz80QJ60as25d
		3idKZdR0IIOm2ta0Lhw/Lz2S1JqvK8ozbTKlEIVYceWSjwIAO///
	    }
	}


	frame $base.mbar -bd 2 -relief raised
	menubutton $base.mbar.file -text File -underline 0 -menu $base.mbar.file.m
	pack $base.mbar.file -side left -padx 5
	set m [menu $base.mbar.file.m]
	#      $m add command -label Open -underline 0 -command "Load $base"
	#      $m add command -label Refresh -underline 0 -command "Refresh $base"
	#      $m add separator
	#      $m add command -label Index -acc Ctrl-i -command "\
		#              HelpViewer::LoadFile $base.h.h [list $file] ;\
		#              set HelpViewer::lastDir [list [file dirname $file]]"
	$m add command -label Refresh -acc Ctrl-r -command "HelpViewer::Refresh $base.h.h"
	$m add separator
	$m add command -label Close -acc Ctrl-q -command "destroy $base"
	bind $base <Control-o> "HelpViewer::Load $base.h.h"
	bind $base <Control-r> "$m invoke Refresh"
	bind $base <Control-i> "$m invoke Index"
	bind $base <Control-q> "$m invoke Close"

	menubutton $base.mbar.edit -text Edit -underline 0 -menu $base.mbar.edit.m
	pack $base.mbar.edit -side left -padx 5
	set m [menu $base.mbar.edit.m]
	$m add command -label Copy -acc Ctrl-c -command "HelpViewer::CopySelected $base.h.h"
	bind $base <Control-c> "$m invoke Copy"

	menubutton $base.mbar.go -text Go -underline 0 -menu $base.mbar.go.m
	pack $base.mbar.go -side left -padx 5
	set History::menu [menu $base.mbar.go.m]
	$History::menu add command -label Backward -acc Alt-<- -state disabled -command \
		"History::GoBackward $base.h.h"
	$History::menu add command -label Forward -acc Alt--> -state disabled -command \
		"History::GoForward $base.h.h"
	bind $base <Alt-Left> "$History::menu invoke Backward"
	bind $base <Alt-Right> "$History::menu invoke Forward"

	global showImages
	set showImages 1
	#    $m add checkbutton -label {Show Images} -variable showImages
	#trace variable showImages w "Refresh $base.h.h"

	# Construct the main HTML viewer
	#
	frame $base.main
	frame $base.h
	html $base.h.h \
		-yscrollcommand "$base.h.vsb set" \
		-xscrollcommand "$base.f2.hsb set" \
		-padx 5 \
		-pady 9 \
		-formcommand HelpViewer::FormCmd \
		-imagecommand HelpViewer::ImageCmd \
		-scriptcommand HelpViewer::ScriptCmd \
		-appletcommand HelpViewer::AppletCmd \
		-underlinehyperlinks 0 \
		-bg white -tablerelief raised \
		-resolvercommand HelpViewer::ResolveUri \
		-exportselection 1 \
		-takefocus 1 \
		-fontcommand HelpViewer::pickFont \
		-width 550 \
		-height 500

	bind $base.h.h.x <1> "HelpViewer::HrefBinding $base.h.h %x %y"

	# Pack the HTML widget into the main screen.
	#
	scrollbar $base.h.vsb -orient vertical -command "$base.h.h yview"
	frame $base.f2
	frame $base.f2.sp -width [winfo reqwidth $base.h.vsb] -bd 2 -relief raised
	pack $base.f2.sp -side right -fill y
	scrollbar $base.f2.hsb -orient horizontal -command "$base.h.h xview"
	pack $base.f2.hsb -side top -fill x
	pack $base.h.vsb -side right -fill y
	pack $base.h.h -side left -fill both -expand 1

	pack $base.f2 -in $base.main -side bottom -fill x
	pack $base.h -in $base.main -side top -fill both -expand 1

	grid $base.mbar -colmun 0 -row 0 -columnspan 3 -sticky ew
	grid $base.main -colmun 2 -row 2 -sticky nsew
	grid columnconf $base 2 -weight 1
	grid rowconf $base  2 -weight 1

	# This procedure is called when the user selects the File/Open
	# menu option.
	#
	set lastDir [pwd]

	#      $base.h.h token handler Frameset "FrameCmd $base"
	#      $base.h.h token handler Frame "FrameCmd $base"
	#      $base.h.h token handler /Frameset "FrameCmd $base"
	

	# This binding changes the cursor when the mouse move over
	# top of a hyperlink.
	#
	bind HtmlClip <Motion> {
	    set parent [winfo parent %W]
	    set url [$parent href %x %y] 
	    if {[string length $url] > 0} {
		$parent configure -cursor hand2
	    } else {
		$parent configure -cursor {}
	    }
	}


	bind $base <4> "\
		$base.h.h yview scroll -3 units ;\
		break"

	bind $base <5> "\
		$base.h.h yview scroll 3 units;\
		break"

	focus $base.h.h

	bind $base.h.h
	bind $base.h.h <Prior> {
	    %W yview scroll -1 pages
	}
	
	bind $base.h.h <Next> {
	    %W yview scroll 1 pages
	}


	bind $base <1> "HelpViewer::ManageSel $base.h.h %W %x %y press"
	bind $base <B1-Motion> "HelpViewer::ManageSel $base.h.h %W %x %y motion"
	bind $base <ButtonRelease-1> "HelpViewer::ManageSel $base.h.h %W %x %y release"
	if { $tcl_platform(platform) != "windows"} {
	    selection handle $base.h.h "HelpViewer::CopySelected $base.h.h"
	}


	# If an arguent was specified, read it into the HTML widget.
	#
	update
	if {$file!=""} {
	    LoadFile $base.h.h $file
	}

    }

}

proc HelpWindowRam { args } {

    set geom ""
    set title ""
    set win .help
    if { [lsearch $args -g*] != -1 } {
	set idx [lsearch $args -g*]
	set geom [lindex $args [expr $idx+1]]
	set args [lreplace $args $idx [expr $idx+1]]
    } 
    if { [lsearch $args -tt*] != -1 } {
	set idx [lsearch $args -tt*]
	set HelpPrefs::tttrick 1
	set args [lreplace $args $idx $idx]
    }
    if { [lsearch $args -ti*] != -1 } {
	set idx [lsearch $args -ti*]
	set title [lindex $args [expr $idx+1]]
	set args [lreplace $args $idx [expr $idx+1]]
    }
    if { [lsearch $args -win*] != -1 } {
	set idx [lsearch $args -win*]
	set win [lindex $args [expr $idx+1]]
	set args [lreplace $args $idx [expr $idx+1]]
    }
    set file $args
    if { [llength $file] == 1 } {
	set file [lindex $file 0]
    }
    HelpViewer::HelpWindow $file $win $geom $title
}

#  if { ![info exists GIDDEFAULT] } {
#      wm withdraw .
#      HelpWindowRam $argv
#      bind .help <Destroy> "exit"
#  }
